﻿using System;
using MccDaq;
using System.IO;
namespace AinScanBackgroundContinuousScan
{
    class Program
    {

        public const int NUMOFBLKS      = 32;
        public const int PACKETSIZE     = 32;
        public const int CHANCOUNT      = 4;
        public const int FIRSTCHANNEL   = 0;
        public const int LASTCHANNEL    = 3;
        public const int FREQ           = 100;

  
        //This program works with several devices, enable the Device string for your product
 
        public const string DEVICE = "USB-1208LS";

        public static StreamWriter fStream;

        public static MccBoard daq;

        static void Main(string[] args)
        {


            int     BoardNum        = 0;
            int     Rate            = FREQ;
            bool    ReadLower       = true;

            Console.WriteLine("\nContinuous Background Scanning Example\n");

            Console.WriteLine("\nLocating board...\n");

            BoardNum = GetBoardNum(DEVICE);


            if (BoardNum == -1)
            {
                Console.WriteLine("No {0} detected!", DEVICE);
                WaitForKey();
                return;
            }


            int BUFFERSIZE = PACKETSIZE * CHANCOUNT * NUMOFBLKS; 
            int HALFBUFFSIZE = BUFFERSIZE / 2;

            daq = new MccDaq.MccBoard( BoardNum );
           
            IntPtr buffer = MccService.WinBufAllocEx( BUFFERSIZE );

            if( buffer == IntPtr.Zero )
            {
                Console.WriteLine( "Bad Handle" );
                return;
            }

            short[] chArray = new short[CHANCOUNT]; //configuration array for channel numbers
            Range[] chRange = new Range[CHANCOUNT]; //configuration array for input ranges

            chArray[0] = 0;
            chArray[1] = 1;
            chArray[2] = 2;
            chArray[3] = 3;

            chRange[0] = Range.Bip10Volts;
            chRange[1] = Range.Bip10Volts;
            chRange[2] = Range.Bip10Volts;
            chRange[3] = Range.Bip10Volts;

            IsError(daq.ALoadQueue(chArray, chRange, CHANCOUNT));

            ScanOptions options  = ScanOptions.Background|ScanOptions.Continuous|ScanOptions.ConvertData;

            //setup the acquisiton
            IsError( daq.AInScan(   FIRSTCHANNEL, 
                                    LASTCHANNEL, 
                                    BUFFERSIZE, 
                                    ref Rate, 
                                    Range.Bip10Volts, 
                                    buffer,
                                    options)
                    );


            fStream = new StreamWriter(@"C:\Users\Public\Documents\DataFile.asc");
            CreateFileHeaders(); //writes basic info to the beginning of the file

            int Count = 0;
            int Index = 0;
            short daqStatus;

            ushort[] ushortArray = new ushort[BUFFERSIZE];

            //Loop until key press
            do{
                IsError(daq.GetStatus( out daqStatus, out Count, out Index, FunctionType.AiFunction ));
                Console.Write("Count={0}, Index={1}\r", Count, Index);

                if ((Index >= HALFBUFFSIZE) & ReadLower) //check for 50% more data
                {
                    //get lower half of buffer
                    IsError(MccService.WinBufToArray(buffer, ushortArray, 0, HALFBUFFSIZE));

                    DisplayData(ushortArray, HALFBUFFSIZE/CHANCOUNT);
                    ReadLower = false; //flag that controls the next read
                }
                else if ((Index < HALFBUFFSIZE) & !ReadLower)
                {
                    //get the upper half
                    IsError( MccService.WinBufToArray(buffer, ushortArray, HALFBUFFSIZE, HALFBUFFSIZE));

                    DisplayData(ushortArray, HALFBUFFSIZE/CHANCOUNT);
                    ReadLower = true;//flag that controls the next read
                }


            } while ( !Console.KeyAvailable );
            ConsoleKeyInfo cki = Console.ReadKey();

            

            //flush any buffered data out to disk
            fStream.Close();

            //stop the  acquisition
            IsError(daq.StopBackground(FunctionType.AiFunction));

            //free up memory
            MccService.WinBufFreeEx(buffer);

            WaitForKey();
            
        }
        public static int IsError(ErrorInfo e)
        {
            if (e.Value != 0)
            {
                Console.WriteLine(e.Message);
                WaitForKey();
                return 1;
            }
            return 0;
        }

        /*////////////////////////////////////////////////////////////////////////////////////*/

        public static int GetBoardNum(string dev)
        {

            for (int BoardNum = 0; BoardNum < 99; BoardNum++)
            {
                MccDaq.MccBoard daq = new MccDaq.MccBoard(BoardNum);
                if (daq.BoardName.Contains(dev))
                {     
                    Console.WriteLine("{0} board number = {1}\n", dev, BoardNum.ToString());
                    return BoardNum;
                }
            }
            return -1;
        }


        /*////////////////////////////////////////////////////////////////////////////////////*/

        public static void WaitForKey()
        {
            Console.WriteLine("\nPress any key to continue...");
            do
            {
                System.Threading.Thread.Sleep(20);

            } while (!Console.KeyAvailable);
            ConsoleKeyInfo cki = Console.ReadKey();

        }

        public static void DisplayData(ushort[] datArray, int rows)
        {
           
            int i = 0;
            float temp = 0;

            for (int row = 0; row < rows; row++)
            {
                
                for (int c = 0; c < CHANCOUNT; c++)
                {

                    IsError(daq.ToEngUnits(Range.Bip10Volts, datArray[i], out temp));
                    Console.Write("{0}\t", temp.ToString("0.000").PadLeft(10));
                    fStream.Write("{0}\t", temp.ToString("0.000"));
                    i++;
                }

                Console.Write("\r\n");
                fStream.Write("\r\n");
            }
            fStream.Flush();
        }
        /*////////////////////////////////////////////////////////////////////////////////////*/

        public static void CreateFileHeaders()
        {
            Console.WriteLine("Reading channels {0} through {1}\n", FIRSTCHANNEL, LASTCHANNEL);

            //''''''''''''''''''''' create text file header strings ''''''''''''''''
            string[] hdr = new string[6];
            hdr[0] = "Recording date     : {0}";
            hdr[1] = "Block length       : {0}";
            hdr[2] = "Number of Packets  : {0}";
            hdr[3] = "Delta              : {0} sec.";
            hdr[4] = "Number of channels : {0}";


            //''''''''''''''''''''''' build column headers string '''''''''''''''''''

            for (int j = FIRSTCHANNEL; j < LASTCHANNEL; j++)
            {
                hdr[5] = hdr[5] + String.Format("In{0} [V]", j);
                hdr[5] = hdr[5] + "\t";

            }
            hdr[4].Trim('\t');

            //'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
            DateTime MyDate = DateTime.Now;

            fStream.WriteLine(hdr[0], MyDate);
            fStream.WriteLine(hdr[1], PACKETSIZE);
            fStream.WriteLine(hdr[2], NUMOFBLKS);
            fStream.WriteLine(hdr[3], (1.0f / FREQ).ToString("0.000"));
            fStream.WriteLine(hdr[4], CHANCOUNT);
            fStream.WriteLine(hdr[5]);

            Console.WriteLine(hdr[0], MyDate);
            Console.WriteLine(hdr[1], PACKETSIZE);
            Console.WriteLine(hdr[2], NUMOFBLKS);
            Console.WriteLine(hdr[3], (1.0f / FREQ).ToString("0.000"));
            Console.WriteLine(hdr[4], CHANCOUNT);
            Console.WriteLine(hdr[5]);

        }

    }
}
